# <trigger>
#	<usage>perl createRelation.pl relationName attributeName relationType</usage>
#	<version>4.3</version>
#	<description>Create a relationship from the CR in the attributeName parameter to the CR that triggered this script.
#					The attributeName parameter contains the name of an attribute on the
#					transition that contains the CR number for the RCR object.
#   </description>
#	<argument1>
#		<name>relation</name>
#		<required>true</required>
#		<type></type>
#		<description>The name of the relationship to create between the CRs</description>
#	</argument1>
#	<argument2>
#		<name>attributeName</name>
#		<required>true</required>
#		<type></type>
#		<description>The name of the attribute that contains the CR number.</description>
#    </argument2>
#	<argument3>
#		<name>rel_type</name>
#		<required>true</required>
#		<type></type>
#		<description>The type of relationship to create.  Possible values are: problem_problem, problem_task, problem_object, task_task, task_object, and object_object</description>
#	</argument3>
# </trigger>
use ChangeSynergy::utf8Conditional;
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;

my $relation = shift;
my $relation_attr = shift;
my $rel_type = shift;

#Create a new TriggerParser object
my $trigger = new ChangeSynergy::TriggerParser($ARGV[-1]);

my %attributes = $trigger->get_attributes();
my $create = $attributes{$relation_attr};

if((!defined($create)) || ("\L$create" eq ""))
{
	exit 0;
}

#Create new csapi object
my $csapi = new ChangeSynergy::csapi();
my $fromObject   = $trigger->get_object_id();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(),
					$trigger->get_token(), $trigger->get_database());

#verify that the target object exists.
#Create a query string and then call the api to get a data report.
my $queryString = "problem_number='" . $create . "'";
my $data = $csapi->QueryData($user, "Basic Summary", $queryString, undef, undef, undef);

if ($data->getDataSize() == 0) {
	die "Unable to create relation, CR " . $create . " does not exist\n";
}

my $tmpstr = $csapi->CreateRelation($user, "FALSE", $create, $fromObject, $relation, $rel_type);
print "Created $relation relationship between $fromObject and $create\n";
exit 0;
