# <trigger>
#	<usage>perl create_RCR_and_SCR_relation.pl attributeName relationName relationType</usage>
#	<version>4.3</version>
#	<description>Create a relationship between an Engineering Change Request and a related Requirements Change Request and its
#					parent System Change Request. The attributeName parameter contains the name of an attribute on the
#					transition	that contains the CR number for the RCR object.
#
#   </description>
#	<argument1>
#		<name>attributeName</name>
#		<required>true</required>
#		<type></type>
#		<description>The name of the attribute that contains the RCR object CR number.</description>
#    </argument1>
#	<argument2>
#		<name>relation</name>
#		<required>true</required>
#		<type></type>
#		<description>The name of the relationship to create between the RCR and IR, and between the SCR and IR</description>
#	</argument2>
#	<argument3>
#		<name>rel_type</name>
#		<required>true</required>
#		<type></type>
#		<description>The type of relationship to create.  Possible values are: problem_problem, problem_task, problem_object, task_task, task_object, and object_object</description>
#	</argument3>
# </trigger>
use ChangeSynergy::utf8Conditional;
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;
use strict;

my $relation_attr = shift;
my $relation = shift;
my $rel_type = shift;

#Create a new TriggerParser object
my $trigger = new ChangeSynergy::TriggerParser($ARGV[-1]);

my %attributes = $trigger->get_attributes();
my $RCRnum = $attributes{$relation_attr};

if((!defined($RCRnum)) || ("\L$RCRnum" eq ""))
{
	exit 0;
}

#Create new csapi object
my $csapi = new ChangeSynergy::csapi();
my $toObject   = $trigger->get_object_id();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(),
					$trigger->get_token(), $trigger->get_database());

#verify that the target object exists.
#Create a query string and then call the api to get a data report.
my $queryString = "problem_number='" . $RCRnum . "'";
my $data;

eval
{
	$data = $csapi->QueryData($user, "Data_Report_rcr_and_scr", $queryString, undef, undef, undef);
};
if ($@)
{
	die $@;
}

if ($data->getDataSize() == 0) {
	die "Unable to create relation, CR " . $RCRnum . " does not exist\n";
}

eval
{
	my $tmpstr = $csapi->CreateRelation($user, "FALSE", $RCRnum, $toObject, $relation, $rel_type);
};

if ($@) {
	die "Successfully submitted CR $toObject, but failed to create relationship between CR $toObject and CR $RCRnum\n";
}

print "Created $relation relationship between $RCRnum and $toObject\n";

my $rcr = $data->getDataObject(0);
my $scr;

eval
{
	$scr = $rcr->getChildData->getDataObject(0);
};

if ($@)
{
	#There is no SCR associated with the RCR, exit now.
	exit 0;
}

my $scrNum = $scr->getDataObjectByName("problem_number")->getValue();

eval
{
	my $tmpstr = $csapi->CreateRelation($user, "FALSE", $scrNum, $toObject, $relation, $rel_type);
};

if ($@) {
	die "Successfully submitted CR $toObject, but failed to create relationship between CR $toObject and CR $scrNum\n";
}

print "Created $relation relationship between $scrNum and $toObject\n";

exit 0;
